@@ -45,6 +45,7 @@ from iqm.iqm_client.models import (
     ClientLibrary,
     ClientLibraryDict,
     DynamicQuantumArchitecture,
+    QIRCode,
     QualityMetricSet,
     QuantumArchitectureSpecification,
     RunCounts,
@@ -64,6 +65,7 @@ import requests
 from requests import HTTPError
 
 from iqm.station_control.client.iqm_server.iqm_server_client import IqmServerClient
+from iqm.station_control.client.qon import ObservationFinder
 from iqm.station_control.client.utils import init_station_control
 from iqm.station_control.interface.models import ObservationLite
 from iqm.station_control.interface.station_control import StationControlInterface
@@ -133,7 +135,7 @@ class IQMClient:
 
         self._station_control: StationControlInterface = init_station_control(
             root_url=url,
-            get_token_callback=self._token_manager.get_bearer_token,
+            get_token_callback=self._token_manager.get_bearer_token,  # type:ignore[arg-type]
             client_signature=client_signature,
         )
         self._api = APIConfig(url)
@@ -232,6 +234,8 @@ class IQMClient:
 
         for i, circuit in enumerate(circuits):
             try:
+                if isinstance(circuit, (QIRCode)):
+                    continue
                 # validate the circuit against the static information in iqm.iqm_client.models._SUPPORTED_OPERATIONS
                 validate_circuit(circuit)
             except ValueError as e:
@@ -553,7 +557,7 @@ class IQMClient:
         calibration_set = quality_metrics.calibration_set
 
         return QualityMetricSet(
-            **{
+            **{  # type:ignore[arg-type]
                 "calibration_set_id": calibration_set.observation_set_id,
                 "calibration_set_dut_label": calibration_set.dut_label,
                 "calibration_set_number_of_observations": len(calibration_set.observation_ids),
@@ -619,9 +623,11 @@ class IQMClient:
 
         return CalibrationSet(
             calibration_set_id=calibration_set.observation_set_id,
-            calibration_set_dut_label=calibration_set.dut_label,
+            calibration_set_dut_label=calibration_set.dut_label,  # type:ignore[arg-type]
             calibration_set_created_timestamp=str(calibration_set.created_timestamp.isoformat()),
-            calibration_set_end_timestamp=str(calibration_set.end_timestamp.isoformat()),
+            calibration_set_end_timestamp=""
+            if calibration_set.end_timestamp is None
+            else str(calibration_set.end_timestamp.isoformat()),
             calibration_set_is_invalid=calibration_set.invalid,
             observations={
                 observation.dut_field: _observation_lite_to_json(observation) for observation in observations
@@ -902,3 +908,33 @@ class IQMClient:
         except json.decoder.JSONDecodeError as e:
             raise EndpointRequestError(f"Invalid response: {response.text}, {e!r}") from e
         return model
+
+    def get_structured_metrics(self, calibration_set_id: UUID | None = None) -> ObservationFinder:
+        """Retrieve the given calibration set and related quality metrics from the server.
+
+        Args:
+            calibration_set_id: ID of the calibration set to retrieve.
+                If ``None``, the current default calibration set is retrieved.
+
+        Returns:
+            Requested calibration set and related quality metrics in a searchable structure.
+
+        Raises:
+            EndpointRequestError: did not understand the endpoint response
+            ClientAuthenticationError: no valid authentication provided
+            HTTPException: HTTP exceptions
+
+        """
+        if isinstance(self._station_control, IqmServerClient):
+            raise ValueError("The get_structured_metrics method is not supported by IqmServerClient.")
+
+        if not calibration_set_id:
+            # find out the default calset id
+            default_calset = self._station_control.get_default_calibration_set()
+            calibration_set_id = default_calset.observation_set_id
+
+        calset_obs = self._station_control.get_observation_set_observations(calibration_set_id)
+        quality_metrics = self._station_control.get_calibration_set_quality_metrics(calibration_set_id)
+        qm_obs = quality_metrics.observations
+
+        return ObservationFinder(list(calset_obs) + qm_obs)
