@@ -459,7 +459,10 @@ class Circuit(BaseModel):
         return instructions
 
 
-CircuitBatch = list[Circuit]
+QIRCode = str
+"""QIR program code in string representation"""
+
+CircuitBatch = list[Circuit | QIRCode]
 """Type that represents a list of quantum circuits to be executed together in a single batch."""
 
 
@@ -473,7 +476,12 @@ def validate_circuit(circuit: Circuit) -> None:
         pydantic.error_wrappers.ValidationError: validation failed
 
     """
-    Circuit.model_validate(circuit.__dict__)
+    if isinstance(circuit, Circuit):
+        Circuit.model_validate(circuit.__dict__)
+    elif isinstance(circuit, QIRCode):
+        pass
+    else:
+        raise ValueError("Every circuit in a batch should be of type <Circuit> or <QIRCode>")
 
 
 class SingleQubitMapping(BaseModel):
@@ -706,6 +714,20 @@ class GateInfo(BaseModel):
         loci_sorted = sorted(loci_set, key=lambda locus: tuple(map(_component_sort_key, locus)))
         return tuple(loci_sorted)
 
+    def get_default_implementation(self, locus: Locus) -> str:
+        """Default implementation of this gate for the given locus.
+
+        Args:
+            locus: gate locus
+
+        Returns:
+            Name of the default implementation of this gate for ``locus``.
+
+        """
+        if (impl := self.override_default_implementation.get(locus)) is not None:
+            return impl
+        return self.default_implementation
+
 
 class DynamicQuantumArchitecture(BaseModel):
     """Dynamic quantum architecture as returned by server.
